package com.isoft.common.sm4;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.*;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Set;

/**
 * sm4参数解密
 */
@Aspect
@Component
public class SM4Aop {

    /**
     * 所有类型传参解密
     * 前台所有参数以json字符串形式加密后，传给名为encrypt的参数，可以是json形式传值，也可以是表单形式
     * rest形式的参数，路径上可以随便写内容以匹配接口，实际值需要拼成json格式
     * 接口中，使用@RequestBody和@RequestParam注释的参数，需要设置默认可不传，required = false
     * 集合类型的参数，需要指定泛型
     * 现只处理了部分类型的参数，如遇到特殊类型的参数可能需要另做处理
     * 前台需要对参数做加密处理，暂不确定前台是如何封装的请求
     * 如果有文件类型，不能使用@RequestBody
     * 文件类型不做处理
     * @param proceedingJoinPoint
     * @return
     * @throws Throwable
     */
    @Around("@annotation(com.isoft.common.sm4.SM4Annotation)")
    public Object decodeSM4(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        // 获取前台的参数
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        // 加密字符串
        String encrypt = request.getParameter("sign");
        // 如果是application/json的传参方式，如：{"encrypt":"xxx..."}
        if (StringUtils.isEmpty(encrypt)) {
            String jsonStr = IOUtils.toString(request.getInputStream(), StandardCharsets.UTF_8);
            if (StringUtils.isNotEmpty(jsonStr))
                encrypt = JSON.parseObject(jsonStr).getString("sign");
        }
        // 获取方法上的所有参数
        Object[] args = proceedingJoinPoint.getArgs();
        if (StringUtils.isNotEmpty(encrypt)) {
            // 解密成json
            //SM4Utils sm4Utils = new SM4Utils();
            Sm sm4Utils = new Sm();
            String jsonStr = sm4Utils.decryptData_ECB(encrypt);
            // 转json对象
            JSONObject jsonObject = JSON.parseObject(jsonStr);
            // 获取此次调用方法
            MethodSignature signature = (MethodSignature) proceedingJoinPoint.getSignature();
            Method method = signature.getMethod();
            // 获取此次调用方法所有参数
            Parameter[] parameters = method.getParameters();
            // 获取参数类型列表
            Type[] parameterTypes = method.getGenericParameterTypes();
            // 循环参数赋值
            for (int i = 0; i < args.length; i++) {
                // 得到方法中默认的参数
                Parameter parameter = parameters[i];
                // 判断参数类型
                Class<?> clazz = parameter.getType();
                // 获得参数名称
                String parameterName = parameter.getName();
                if (this.isBaseType(clazz)) {
                    // 如果是基本类型，直接赋值
                    args[i] = jsonObject.get(parameterName);
                } else if (clazz.equals(BigDecimal.class)) {
                    // 大数字类型特殊处理
                    Object o = jsonObject.get(parameterName);
                    if (o != null)
                        args[i] = new BigDecimal(o.toString());
                } else if (clazz.equals(List.class) || clazz.equals(Set.class)) {
                    // 集合类型，没测试set类型，感觉没问题
                    // 获取对应位置的参数
                    ParameterizedType parameterType = (ParameterizedType) parameterTypes[i];
                    // 获取集合类型的泛型类型
                    Class realClazz = (Class) parameterType.getActualTypeArguments()[0];
                    // 先从json中获取对应的数组对象
                    JSONArray jsonArray = jsonObject.getJSONArray(parameterName);
                    // 转成对应的java类型
                    args[i] = JSON.parseArray(jsonArray.toJSONString(), realClazz);
                } else if (clazz.equals(MultipartFile.class) || clazz.equals(MultipartFile[].class)) {
                    // 文件类型不做加解密处理
//                    System.out.println(args[i].toString());
                } else {
                    // 如果是对象类型，json转换
                    args[i] = JSON.parseObject(jsonStr, clazz);
                }
            }
        }
        return proceedingJoinPoint.proceed(args);
    }


    /**
     * 判断object是否为基本类型
     * @param clazz
     * @return
     */
    public static boolean isBaseType(Class<?> clazz) {
        if (clazz.equals(java.lang.Integer.class) || clazz.equals(int.class) ||
                clazz.equals(java.lang.Byte.class) || clazz.equals(byte.class) ||
                clazz.equals(java.lang.Long.class) || clazz.equals(long.class) ||
                clazz.equals(java.lang.Double.class) || clazz.equals(double.class) ||
                clazz.equals(java.lang.Float.class) || clazz.equals(float.class) ||
                clazz.equals(java.lang.Character.class) || clazz.equals(char.class) ||
                clazz.equals(java.lang.Short.class) || clazz.equals(short.class) ||
                clazz.equals(java.lang.Boolean.class) || clazz.equals(boolean.class) ||
                clazz.equals(java.lang.String.class)
        )
            return true;
        return false;
    }

}
