package com.isoft.common.utils.file;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Random;

import com.isoft.common.utils.NumberUtil;
import com.isoft.project.util.FilesUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;
import com.isoft.common.constant.Constants;
import com.isoft.common.exception.file.FileNameLengthLimitExceededException;
import com.isoft.common.exception.file.FileSizeLimitExceededException;
import com.isoft.common.exception.file.InvalidExtensionException;
import com.isoft.common.utils.DateUtils;
import com.isoft.common.utils.StringUtils;
import com.isoft.common.utils.security.Md5Utils;
import com.isoft.framework.config.IsoftConfig;

import static com.isoft.project.upload.util.FileFFMp4.inputStreamToFile;

/**
 * 文件上传工具类
 * 
 * @author isoft
 */
@Component
public class FileUploadUtils
{


    /**
     * 默认大小 50M
     */
    private static String fileSize;
    @Value("${upload.size}")
    public void setFileSize(String size) {
        FileUploadUtils.fileSize = size;
    }

    /**
     * 默认的文件名最大长度 100
     */
    private static String fileLength;
    @Value("${upload.length}")
    public void setFileLength(String length) {
        FileUploadUtils.fileLength = length;
    }



//    public static final long DEFAULT_MAX_SIZE = Long.parseLong(size);
//
//    /**
//     * 默认的文件名最大长度 100
//     */
//    public static final int DEFAULT_FILE_NAME_LENGTH =Integer.parseInt(length);

    /**
     * 默认上传的地址
     */
    private static String defaultBaseDir = IsoftConfig.getProfile();

    private static int counter = 0;

    public static void setDefaultBaseDir(String defaultBaseDir)
    {
        FileUploadUtils.defaultBaseDir = defaultBaseDir;
    }

    public static String getDefaultBaseDir()
    {
        return defaultBaseDir;
    }

    /**
     * 以默认配置进行文件上传
     *
     * @param file 上传的文件
     * @return 文件名称
     * @throws Exception
     */
    public static final String upload(MultipartFile file,String type) throws IOException
    {
        try
        {
            return upload(getDefaultBaseDir(), file, MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION,type);
        }
        catch (Exception e)
        {
            throw new IOException(e.getMessage(), e);
        }
    }

    /**
     * 根据文件路径上传
     *
     * @param baseDir 相对应用的基目录
     * @param file 上传的文件
     * @param type 0明文上传，1密文上传
     * @return 文件名称
     * @throws IOException
     */
    public static final String upload(String baseDir, MultipartFile file ,String type) throws IOException
    {
        try
        {
            return upload(baseDir, file, MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION, type);
        }
        catch (Exception e)
        {
            throw new IOException(e.getMessage(), e);
        }
    }

    /**
     * 文件上传
     *
     * @param baseDir 相对应用的基目录
     * @param file 上传的文件
     * @param allowedExtension 上传文件类型
     * @param type 0明文上传，1密文上传
     * @return 返回上传成功的文件名
     * @throws FileSizeLimitExceededException 如果超出最大大小
     * @throws FileNameLengthLimitExceededException 文件名太长
     * @throws IOException 比如读写文件出错时
     * @throws InvalidExtensionException 文件校验异常
     */
    public static final String upload(String baseDir, MultipartFile file, String[] allowedExtension,String type)
            throws FileSizeLimitExceededException, IOException, FileNameLengthLimitExceededException,
            InvalidExtensionException
    {
        int fileNameLength = file.getOriginalFilename().length();
        if (fileNameLength > Integer.parseInt(FileUploadUtils.fileLength))
        {
            throw new FileNameLengthLimitExceededException(Integer.parseInt(FileUploadUtils.fileLength));
        }
        //判断文件大小及类型
        assertAllowed(file, allowedExtension);
        //根据文件内容，生成文件唯一名称
        String fileName = extractFilename(file,type);
        //创建文件路径
        File desc = getAbsoluteFile(baseDir, fileName);
        //写入文件
        file.transferTo(desc);
        String pathFileName = getPathFileName(baseDir, fileName);
        return pathFileName;
    }

    /**
     ** @param type 0明文上传，1密文上传
     * 获取文件的Md5值, 编码文件名
     */
    public static final String extractFilename(MultipartFile file,String type)
    {
        String extension = getExtension(file);
        //MultipartFile 转 File
        File toFile = null;
        InputStream ins = null;
        String s = null;
        try {
            ins = file.getInputStream();
            toFile = new File(file.getOriginalFilename());
            inputStreamToFile(ins, toFile);
            ins.close();
            InputStream in = new FileInputStream(toFile);
            //获取文件的Md5唯一值
            s = DigestUtils.md5Hex(in);
            in.close();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (toFile.exists())
                FilesUtils.delDir(file.getOriginalFilename());
        }

        String fileName = null;
        if (type.equals("1")){
            //相对密文路径
            String[] split = DateUtils.datePath().split("/");
            fileName = NumberUtil.encrypt(split[0]) +"/"
                    + NumberUtil.encrypt(split[1]) +"/"
                    + NumberUtil.encrypt(split[2]) +"/"
                    + s + "." + extension;
        }else {
            //相对明文路径
            fileName = DateUtils.datePath() + "/" + s + "." + extension;
        }
        return fileName;
    }


    public static String getRandomString(String str) {
        int sum = 0;
        for (int i = 0; i < str.length(); i++) {
            sum = sum + (int) str.charAt(i);
        }
        return str + sum;

//        Random random = new Random();
//        StringBuilder sb = new StringBuilder();
//        for (int i = 0; i < length; i++) {
//            int number = random.nextInt(str.length());
//            sb.append(str.charAt(number));
//        }
//        return sb.toString();
    }







    private static final File getAbsoluteFile(String uploadDir, String fileName) throws IOException
    {
        File desc = new File(uploadDir + File.separator + fileName);

        if (!desc.getParentFile().exists())
        {
            desc.getParentFile().mkdirs();
        }
        if (!desc.exists())
        {
            desc.createNewFile();
        }
        return desc;
    }

    private static final String getPathFileName(String uploadDir, String fileName) throws IOException
    {
        int dirLastIndex = IsoftConfig.getProfile().length() + 1;
        String currentDir = StringUtils.substring(uploadDir, dirLastIndex);
//        String pathFileName = Constants.RESOURCE_PREFIX + "/" + currentDir + "/" + fileName;
        String pathFileName = fileName;
        return pathFileName;
    }

    /**
     * 编码文件名
     */
    private static final String encodingFilename(String fileName)
    {
        fileName = fileName.replace("_", " ");
        fileName = Md5Utils.hash(fileName );

        return fileName;
    }

    /**
     * 文件大小校验
     *
     * @param file 上传的文件
     * @return
     * @throws FileSizeLimitExceededException 如果超出最大大小
     * @throws InvalidExtensionException
     */
    public static final void assertAllowed(MultipartFile file, String[] allowedExtension)
            throws FileSizeLimitExceededException, InvalidExtensionException
    {
        long size = file.getSize();
        if (Long.parseLong(FileUploadUtils.fileSize) != -1 && size > Long.parseLong(FileUploadUtils.fileSize))
        {
            throw new FileSizeLimitExceededException(Long.parseLong(FileUploadUtils.fileSize) / 1024 / 1024);
        }

        String fileName = file.getOriginalFilename();
        String extension = getExtension(file);
        if (allowedExtension != null && !isAllowedExtension(extension, allowedExtension))
        {
            if (allowedExtension == MimeTypeUtils.IMAGE_EXTENSION)
            {
                throw new InvalidExtensionException.InvalidImageExtensionException(allowedExtension, extension,
                        fileName);
            }
            else if (allowedExtension == MimeTypeUtils.FLASH_EXTENSION)
            {
                throw new InvalidExtensionException.InvalidFlashExtensionException(allowedExtension, extension,
                        fileName);
            }
            else if (allowedExtension == MimeTypeUtils.MEDIA_EXTENSION)
            {
                throw new InvalidExtensionException.InvalidMediaExtensionException(allowedExtension, extension,
                        fileName);
            }
            else
            {
                throw new InvalidExtensionException(allowedExtension, extension, fileName);
            }
        }

    }

    /**
     * 判断MIME类型是否是允许的MIME类型
     *
     * @param extension
     * @param allowedExtension
     * @return
     */
    public static final boolean isAllowedExtension(String extension, String[] allowedExtension)
    {
        for (String str : allowedExtension)
        {
            if (str.equalsIgnoreCase(extension))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取文件名的后缀
     * 
     * @param file 表单文件
     * @return 后缀名
     */
    public static final String getExtension(MultipartFile file)
    {
        String extension = FilenameUtils.getExtension(file.getOriginalFilename());
        if (StringUtils.isEmpty(extension))
        {
            extension = MimeTypeUtils.getExtension(file.getContentType());
        }
        return extension;
    }
}