package com.isoft.project.upload.util;

import com.amazonaws.ClientConfiguration;
import com.amazonaws.Protocol;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3Client;
import com.amazonaws.services.s3.S3ClientOptions;
import com.amazonaws.services.s3.model.*;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.InputStream;
import java.util.List;

/**
 * 迅达云对象存储工具类
 * 与 AWS 的 S3 兼容
 */
@Component
public class S3Util implements InitializingBean {

    @Value("${s3.endpoint}")
    private String endpoint;
    @Value("${s3.accessKey}")
    private String accessKey;
    @Value("${s3.secretKey}")
    private String secretKey;

    private static AmazonS3 s3;

    private S3Util() {

    }

    /**
     * 单例
     */
    private static class S3UtilHolder {
        private static final S3Util singleton = new S3Util();
    }

    /**
     * 获取实例
     * @return
     */
    public static S3Util getInstance() {
        return S3UtilHolder.singleton;
    }

    /**
     * 实例化 S3 连接
     */
    @Override
    public void afterPropertiesSet() {
        AWSCredentials credentials = new BasicAWSCredentials(this.accessKey, this.secretKey);
        ClientConfiguration clientConfig = new ClientConfiguration();
        clientConfig.setSignerOverride("S3SignerType");
        clientConfig.setProtocol(Protocol.HTTP);
        s3 = new AmazonS3Client(credentials, clientConfig);
        s3.setS3ClientOptions(S3ClientOptions.builder().setPathStyleAccess(true).build());
        s3.setEndpoint(this.endpoint);
    }

    /**
     * 获取AmazonS3对象
     * @return 如有需要自己获取AmazonS3对象调用其中方法
     */
    public AmazonS3 getS3() {
        return s3;
    }

    /**
     * 创建桶
     * @param bucketName 桶名称
     * @return 桶对象
     */
    public Bucket create(String bucketName) {
        return s3.createBucket(bucketName);
    }

    /**
     * 获取桶列表
     * @return 桶集合
     */
    public List<Bucket> buckets() {
        return s3.listBuckets();
    }

    /**
     * 上传文件
     * @param bucketName 桶名称
     * @param fileName 上传到桶内的名称
     * @param file 待上传文件
     * @param folder 文件夹路径，多个间用“/”分隔，如 aaa aaa/bbb，可为空
     * @return 返回值不为空可以认为上传成功
     */
    public PutObjectResult upload(String bucketName, String fileName, File file, String folder) {
        // 设置公共可读
        return s3.putObject(new PutObjectRequest(bucketName, this.path(fileName, folder), file).withCannedAcl(CannedAccessControlList.PublicRead));
    }

    /**
     * 直接以流上传
     * @param bucketName
     * @param fileName
     * @param input
     * @param folder
     * @return
     */
    public PutObjectResult upload(String bucketName, String fileName, InputStream input, String folder) {
        // 设置公共可读
        return s3.putObject(new PutObjectRequest(bucketName, this.path(fileName, folder), input, null).withCannedAcl(CannedAccessControlList.PublicRead));
    }

    /**
     * 下载文件
     * @param bucketName 桶名称
     * @param fileName 桶内文件名称
     * @return 返回对象的getObjectContent()方法拿到文件流
     */
    public S3Object download(String bucketName, String fileName) {
        return s3.getObject(new GetObjectRequest(bucketName, fileName));
    }

    /**
     * 获取桶内对象列表
     * @param bucketName 桶名称
     * @return S3ObjectSummary集合，其中key属性为文件名称
     */
    public List<S3ObjectSummary> summary(String bucketName) {
        return summary(bucketName, null);
    }

    /**
     * 获取桶内对象列表
     * @param bucketName 桶名称
     * @param prefix 前缀，可为空
     * @return S3ObjectSummary集合，其中key属性为文件名称
     */
    public List<S3ObjectSummary> summary(String bucketName, String prefix) {
        ListObjectsRequest request = new ListObjectsRequest().withBucketName(bucketName);
        if (prefix != null) {
            request.setPrefix(prefix);
        }
        ObjectListing objectListing = s3.listObjects(request);
        if (objectListing != null) {
            return objectListing.getObjectSummaries();
        } else {
            return null;
        }
    }

    /**
     * 删除桶内对象
     * @param bucketName 桶名称
     * @param fileName 桶内文件名称
     */
    public void delete(String bucketName, String fileName) {
        s3.deleteObject(bucketName, fileName);
    }

    /**
     * 删除桶
     * @param bucketName 桶名称
     */
    public void destory(String bucketName) {
        s3.deleteBucket(bucketName);
    }

    /**
     * 拼接文件夹路径
     * @param fileName
     * @param folder
     * @return
     */
    private String path(String fileName, String folder) {
        if (folder != null && !folder.equals("")) {
            // 不能以 / 开始，会报错
            if (folder.startsWith("/")) {
                folder = folder.substring(1);
            }
            // 需要以 / 结束，表示文件夹路径结束
            if (!folder.endsWith("/")) {
                folder = folder + "/";
            }
            // 将文件夹路径拼到文件名称前面，会自动生成文件夹
            fileName = folder + fileName;
        }
        return fileName;
    }

    public static void main(String[] args) {
        S3Util instance = S3Util.getInstance();
        instance.summary("cybucket");
    }

}
